@@ -174,11 +174,11 @@ wald.fct 		<- function(
   coef.hat		<- coef.est[start:end]
   vcov.hat		<- vcov.est[start:end, start:end]
 
-  if(estimation == "onestep"){
+  if(object$data$estimation == "onestep"){
 #    w.stat		<- n*crossprod(coef.hat, tcrossprod(MASS::ginv(vcov.hat), t(coef.hat)) ) *
 #				(as.vector(crossprod(do.call(res.1s_temp, what = "c"), do.call(Szero.j, what = "c"), na.rm = TRUE) /(n*Time - sum(n.inst)+7)))						#[M:] Stata results with different dof-correction
     w.stat		<- n * Matrix::crossprod(coef.hat, Matrix::tcrossprod(MASS::ginv(as.matrix(vcov.hat)), Matrix::t(coef.hat)) ) *
-				(as.vector(Matrix::crossprod(do.call(Szero.j, what = "c"), do.call(Szero.j, what = "c"), na.rm = TRUE)) /(sum(!is.na(dat.na[, varname.y])) - sum(n.inst)))		#[M:] Adjusted dof-correction (for missing observations)
+				(as.vector(Matrix::crossprod(do.call(Szero.j, what = "c"))) /(sum(!is.na(dat.na[, varname.y])) - sum(n.inst)))		#[M:] Adjusted dof-correction (for missing observations)
   } else{
     w.stat		<- crossprod(coef.hat, tcrossprod(MASS::ginv(as.matrix(vcov.hat)), t(coef.hat)) )
   }
@@ -319,27 +319,35 @@ jtest.fct		<- function(
     stop("Use only with \"pdynmc\" objects.")
   }
 
-  coef.est		<- ifelse((sapply(get(paste("step", object$iter, sep = ""), object$par.optim), FUN = is.na)), yes = get(paste("step", object$iter, sep = ""), object$par.clForm), no = get(paste("step", object$iter, sep = ""), object$par.optim) )
-  Szero.j		<- get(paste("step", object$iter, sep = ""), object$residuals.int)
-  Z.temp		<- object$data$Z.temp
-  W.j			<- get(paste("step", object$iter, sep = ""), object$w.mat)
-  n.inst		<- object$data$n.inst
+  coef.est	    <- ifelse((sapply(get(paste("step", object$iter, sep = ""), object$par.optim), FUN = is.na)), yes = get(paste("step", object$iter, sep = ""), object$par.clForm), no = get(paste("step", object$iter, sep = ""), object$par.optim) )
+  Szero.j		    <- get(paste("step", object$iter, sep = ""), object$residuals.int)
+  Z.temp		    <- object$data$Z.temp
+  stderr.type   <- object$data$stderr.type
+  n.inst		    <- object$data$n.inst
+
+  if(object$data$estimation == "onestep" && stderr.type == "corrected"){
+    W.j			  <- object$w.mat[[2]]
+    warning("Hansen J-Test statistic is inconsistent when error terms are non-spherical.")
+  } else{
+    W.j			  <- get(paste("step", object$iter, sep = ""), object$w.mat)
+    Szero.j		<- get(paste("step", object$iter, sep = ""), object$residuals.int)
+  }
 
 
   K.tot			<- length(coef.est)
-  N				<- length(do.call(what = "c", Szero.j))
+  N				  <- length(do.call(what = "c", Szero.j))
   tzu				<- as.numeric(Reduce("+", mapply(function(x,y) Matrix::crossprod(x,y), Z.temp, Szero.j, SIMPLIFY = FALSE)))
-  stat			<- as.numeric(crossprod(tzu, t(crossprod(tzu, W.j))))
+  stat			<- as.numeric(Matrix::crossprod(tzu, Matrix::crossprod(W.j, tzu)))
   names(stat)		<- "chisq"
-  p				<- sum(n.inst)
-  param			<- p - K.tot
-  names(param)		<- "df"
-  method			<- "J-Test of Hansen"
-  pval			<- stats::pchisq(stat, df = param, lower.tail = FALSE)
-  jtest			<- list(statistic = stat, p.value = pval, parameter = param, method = method
-					,data.name = paste(object$iter, "step GMM Estimation", sep = "")
-					,alternative = paste("overidentifying restrictions invalid", sep = "")
-					)
+  p				      <- sum(n.inst)
+  param			    <- p - K.tot
+  names(param)	<- "df"
+  method			  <- "J-Test of Hansen"
+  pval			    <- stats::pchisq(stat, df = param, lower.tail = FALSE)
+  jtest			    <- list(statistic = stat, p.value = pval, parameter = param, method = method
+					            ,data.name = paste(object$iter, "step GMM Estimation", sep = "")
+					            ,alternative = paste("overidentifying restrictions invalid", sep = "")
+					            )
   class(jtest)		<- "htest"
   return(jtest)
 }
@@ -362,6 +370,121 @@ jtest.fct		<- function(
 
 
 
+#' Sargan test.
+#'
+#' \code{sargan.fct} tests the validity of the overidentifying restrictions.
+#'
+#' The null hypothesis is that the overidentifying restrictions are valid.
+#'    The test statistic is computed as proposed by
+#'    \insertCite{Sar1958estimation;textual}{pdynmc}. As noted by
+#'    \insertCite{Bow2002testing;textual}{pdynmc} and
+#'    \insertCite{Win2005;textual}{pdynmc},
+#'    the test statistic is weakened by many instruments and inconsistent
+#'    in the presence of heteroscedasticity according to
+#'    \insertCite{Roo2009StJ;textual}{pdynmc}.
+#'
+#' @param object An object of class `pdynmc`.
+#' @return An object of class `htest` which contains the Sargan test statistic
+#'    and corresponding p-value for the null hypothesis that the overidentifying
+#'    restrictions are valid.
+#'
+#' @export
+#' @importFrom Matrix crossprod
+#' @importFrom stats pchisq
+#' @importFrom Rdpack reprompt
+#'
+#' @seealso
+#'
+#' \code{\link{pdynmc}} for fitting a linear dynamic panel data model.
+#'
+#' @references
+#'
+#' \insertAllCited{}
+#'
+#'
+#' @examples
+#' ## Load data
+#' data(ABdata, package = "pdynmc")
+#' dat <- ABdata
+#' dat[,c(4:7)] <- log(dat[,c(4:7)])
+#' dat <- dat[c(140:0), ]
+#'
+#' ## Code example
+#' m1 <- pdynmc(dat = dat, varname.i = "firm", varname.t = "year",
+#'     use.mc.diff = TRUE, use.mc.lev = FALSE, use.mc.nonlin = FALSE,
+#'     include.y = TRUE, varname.y = "emp", lagTerms.y = 2,
+#'     fur.con = TRUE, fur.con.diff = TRUE, fur.con.lev = FALSE,
+#'     varname.reg.fur = c("wage", "capital", "output"), lagTerms.reg.fur = c(1,2,2),
+#'     include.dum = TRUE, dum.diff = TRUE, dum.lev = FALSE, varname.dum = "year",
+#'     w.mat = "iid.err", std.err = "corrected", estimation = "onestep",
+#'     opt.meth = "none")
+#' sargan.fct(m1)
+#'
+#' \donttest{
+#' ## Load data
+#'  data(ABdata, package = "pdynmc")
+#'  dat <- ABdata
+#'  dat[,c(4:7)] <- log(dat[,c(4:7)])
+#'
+#' ## Further code example
+#'  m1 <- pdynmc(dat = dat, varname.i = "firm", varname.t = "year",
+#'     use.mc.diff = TRUE, use.mc.lev = FALSE, use.mc.nonlin = FALSE,
+#'     include.y = TRUE, varname.y = "emp", lagTerms.y = 2,
+#'     fur.con = TRUE, fur.con.diff = TRUE, fur.con.lev = FALSE,
+#'     varname.reg.fur = c("wage", "capital", "output"), lagTerms.reg.fur = c(1,2,2),
+#'     include.dum = TRUE, dum.diff = TRUE, dum.lev = FALSE, varname.dum = "year",
+#'     w.mat = "iid.err", std.err = "corrected", estimation = "onestep",
+#'     opt.meth = "none")
+#'  sargan.fct(m1)
+#' }
+#'
+#'
+sargan.fct 		<- function(
+    object
+){
+
+  if(all(class(object) != "pdynmc")) stop("Object needs to be of class 'pdynmc'")
+
+  estimation  <- object$data$estimation
+  coef.est		<- ifelse((sapply(get(paste("step", object$iter, sep = ""), object$par.optim), FUN = is.na)), yes = get(paste("step", object$iter, sep = ""), object$par.clForm), no = get(paste("step", object$iter, sep = ""), object$par.optim) )
+  Szero.j     <- get(paste("step", object$iter, sep = ""), object$residuals.int)
+  Z.temp      <- object$data$Z.temp
+  n.inst		  <- object$data$n.inst
+
+  W.j			    <- object$w.mat[[1]]
+
+  K.tot		  <- length(coef.est)
+  N			    <- length(do.call(what = "c", Szero.j))
+  tzu			  <- Reduce("+", mapply(function(x,y) Matrix::crossprod(x,y), Z.temp, Szero.j, SIMPLIFY = FALSE))
+  stat		  <- as.numeric(Matrix::crossprod(tzu, Matrix::crossprod(W.j, tzu)))
+  names(stat)	  <- "chisq"
+  p			        <- sum(n.inst)
+  param		      <- p - K.tot
+  names(param)	<- "df"
+  method		    <- "Sargan Test"
+  pval		      <- stats::pchisq(stat, df = param, lower.tail = FALSE)
+  sargan		    <- list(statistic = stat, p.value = pval, parameter = param, method = method,
+                      data.name = paste(object$iter, "step GMM Estimation; H0: overidentifying restrictions valid", sep = "")
+  )
+  class(sargan)	<- "htest"
+  return(sargan)
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
@@ -453,59 +576,75 @@ mtest.fct 		<- function(
     stop("Use only with \"pdynmc\" objects.")
   }
 
-  estimation	<- object$data$estimation
-  Szero.j			<- get(paste("step", object$iter, sep = ""), object$residuals.int)
-  Z.temp			<- object$data$Z.temp
-  vcov.est		<- get(paste("step", object$iter, sep = ""), object$vcov)
-  W.j				  <- get(paste("step", object$iter, sep = ""), object$w.mat)
+  estimation    <- object$data$estimation
+  Szero.j       <- get(paste("step", object$iter, sep = ""), object$residuals.int)
+  Z.temp        <- object$data$Z.temp
+  vcov.est      <- get(paste("step", object$iter, sep = ""), object$vcov)
+  W.j           <- get(paste("step", object$iter, sep = ""), object$w.mat)
+  stderr.type   <- object$data$stderr.type
+  std.err       <- get(paste("step", object$iter, sep = ""), object$stderr)
+#  n.inst        <- object$data$n.inst
+  varname.y     <- object$data$varname.y
+  varname.reg   <- object$data$varnames.reg
+  varname.dum   <- object$data$varnames.dum
+  dat.clF.temp  <- rapply(lapply(object$dat.clF, FUN = as.matrix), function(x) ifelse(is.na(x), 0, x), how = "replace")
+  n.obs				  <- nrow(object$data$dat.na) - sum(is.na(object$data$dat.na[, varname.y]))
+  u.hat.m_o     <- lapply(Szero.j, function(x) c(rep(0, times = order), x[1:(length(x) - order)]))
+  tZX           <- Reduce("+", mapply(function(x, y) Matrix::crossprod(x,y), Z.temp, dat.clF.temp, SIMPLIFY = FALSE))
 
-  stderr.type	<- object$data$stderr.type
-  std.err			<- get(paste("step", object$iter, sep = ""), object$stderr)
-  n.inst			<- object$data$n.inst
-  n				    <- object$data$n
-  Time				    <- object$data$Time
-  varname.y			<- object$data$varname.y
-  varname.reg		<- object$data$varnames.reg
-  varname.dum		<- object$data$varnames.dum
-  dat.clF.temp	<- rapply(lapply(object$dat.clF, FUN = as.matrix), function(x) ifelse(is.na(x), 0, x), how = "replace")
-  dat.na			  <- object$data$dat.na
+  if(estimation == "onestep"){
 
+    if(stderr.type == "unadjusted"){
+      H_i           <- 0.5*object$H_i * as.numeric((1/(n.obs - length(object$coefficients))) * Reduce("+", mapply(function(x) Matrix::crossprod(x,x), Szero.j, SIMPLIFY = FALSE)))
+      A             <- object$w.mat[[1]]
+      M.inv         <- solve(Matrix::crossprod(tZX, Matrix::crossprod(A, tZX)))
 
+      tu.m_oHu.m_o  <- Reduce("+", mapply(function(x) Matrix::crossprod(x, Matrix::crossprod(H_i, x)), u.hat.m_o, SIMPLIFY = FALSE))
+      tZHu.m_o      <- Reduce("+", mapply(function(x, y) Matrix::crossprod(x, Matrix::crossprod(H_i, y)), Z.temp, u.hat.m_o, SIMPLIFY = FALSE))
+    }
 
-  K.t			<- length(varname.dum) - length(varname.dum[!(varname.dum %in% varname.reg)])
-  u.hat.m_o		<- lapply(Szero.j, function(x) c(rep(0, times = order), x[1:(length(x)-order)]) )
+    if(stderr.type == "corrected"){
+      H_1i          <- 0.5*object$H_i * as.numeric((1/(n.obs - length(object$coefficients))) * Reduce("+", mapply(function(x) Matrix::crossprod(x,x), Szero.j, SIMPLIFY = FALSE)))
+      A             <- object$w.mat[[1]]
+      M.inv         <- solve(Matrix::crossprod(tZX, Matrix::crossprod(A, tZX)))
+      H_i           <- Reduce("+", mapply(function(x) Matrix::tcrossprod(x,x), Szero.j, SIMPLIFY = FALSE)) * as.numeric((1/(n.obs - length(object$coefficients))) * Reduce("+", mapply(function(x) Matrix::crossprod(x,x), Szero.j, SIMPLIFY = FALSE)))
+      A_2N          <- object$w.mat[[2]]
+
+      tu.m_oHu.m_o  <- Reduce("+", mapply(function(x,y) Matrix::crossprod(x, Matrix::crossprod(Matrix::tcrossprod(y,y), x)), u.hat.m_o, Szero.j, SIMPLIFY = FALSE))
+      tZHu.m_o      <- Reduce("+", mapply(function(x,y,z) Matrix::crossprod(x, Matrix::crossprod(Matrix::tcrossprod(z,z), y)), Z.temp, u.hat.m_o, Szero.j, SIMPLIFY = FALSE))
+      }
 
-  if(estimation == "onestep" & stderr.type == "unadjusted"){
-    #    uHtu			<- lapply(lapply(Szero.j, function(x) crossprod(x,x)), function(x) as.numeric(x) * 0.2* H_i.temp * (1/ (n*T - sum(n.inst)+3) ))
-    uHtu			<- lapply(lapply(Szero.j, function(x) crossprod(x,x)), function(x) as.numeric(x) * H_i * (1/ (sum(!is.na(dat.na[, varname.y])) - sum(n.inst)) ))
-    tu_m_outuu.m_o	<- Reduce("+", mapply(function(x,y) Matrix::crossprod(x, Matrix::tcrossprod(y, Matrix::t(x))), u.hat.m_o, uHtu, SIMPLIFY = FALSE ))
-    tZutuu.m_o		<- Reduce("+", mapply(function(x,y,z) Matrix::tcrossprod(Matrix::crossprod(x,y), Matrix::t(z)), Z.temp, uHtu, u.hat.m_o, SIMPLIFY = FALSE))
-  } else{
-    tu_m_outuu.m_o	<- Reduce("+", mapply(function(x,y) tcrossprod(crossprod(y,x), crossprod(y,x)), Szero.j, u.hat.m_o, SIMPLIFY = FALSE))
-    tZutuu.m_o		<- Reduce("+", mapply(function(x,y,z) Matrix::tcrossprod(Matrix::crossprod(x,y), Matrix::crossprod(z,y)), Z.temp, Szero.j, u.hat.m_o, SIMPLIFY = FALSE))
   }
 
-  tu.m_oX		<- Reduce("+", mapply(function(x,y) Matrix::crossprod(x,y), u.hat.m_o, dat.clF.temp, SIMPLIFY = FALSE))
-  tZX			<- Reduce("+", mapply(function(x,y) Matrix::crossprod(x,y), Z.temp, dat.clF.temp, SIMPLIFY = FALSE))
+  if(estimation != "onestep"){
+    H_i             <- Reduce("+", mapply(function(x) Matrix::tcrossprod(x,x), Szero.j, SIMPLIFY = FALSE))
+    A               <- W.j
+    M.inv           <- solve(Matrix::crossprod(tZX, Matrix::crossprod(A, tZX)))
 
-  frac.num		<- Reduce("+", mapply(function(x,y) crossprod(x,y), Szero.j, u.hat.m_o, SIMPLIFY = FALSE))
-  frac.denom.sq	<- (as.numeric(tu_m_outuu.m_o - 2* Matrix::crossprod(Matrix::t(tu.m_oX), Matrix::crossprod(Matrix::t(vcov.est), Matrix::crossprod(tZX, Matrix::tcrossprod(W.j, Matrix::t(tZutuu.m_o))))) + Matrix::crossprod(Matrix::t(tu.m_oX), Matrix::tcrossprod(vcov.est, tu.m_oX))))
+    tu.m_oHu.m_o    <- Reduce("+", mapply(function(x,y) Matrix::crossprod(x, Matrix::crossprod(Matrix::tcrossprod(y,y), x)), u.hat.m_o, Szero.j, SIMPLIFY = FALSE))
+    tZHu.m_o        <- Reduce("+", mapply(function(x,y,z) Matrix::crossprod(x, Matrix::crossprod(Matrix::tcrossprod(z,z), y)), Z.temp, u.hat.m_o, Szero.j, SIMPLIFY = FALSE))
+  }
 
-  if(frac.denom.sq < 0){
-    frac.denom	<- sqrt(abs(as.numeric(tu_m_outuu.m_o - 2* Matrix::crossprod(Matrix::t(tu.m_oX), Matrix::crossprod(Matrix::t(vcov.est), Matrix::crossprod(tZX, Matrix::tcrossprod(W.j, Matrix::t(tZutuu.m_o))))) + Matrix::crossprod(Matrix::t(tu.m_oX), Matrix::tcrossprod(vcov.est, tu.m_oX)))))
+  tu.m_oX         <- Reduce("+", mapply(function(x, y) Matrix::crossprod(x, y), u.hat.m_o, dat.clF.temp, SIMPLIFY = FALSE))
+  frac.num        <- Reduce("+", mapply(function(x, y) crossprod(x, y), u.hat.m_o, Szero.j, SIMPLIFY = FALSE))
+  frac.denom.sq   <- (as.numeric(tu.m_oHu.m_o -
+                                 2 * Matrix::crossprod(Matrix::t(tu.m_oX),
+                                                       Matrix::crossprod(M.inv, Matrix::crossprod(tZX, Matrix::tcrossprod(A, Matrix::t(tZHu.m_o))))) +
+                                 Matrix::crossprod(Matrix::t(tu.m_oX), Matrix::tcrossprod(vcov.est, tu.m_oX))))
+  if (frac.denom.sq < 0) {
+    frac.denom <- sqrt(abs(frac.denom.sq))
     warning("Absolute value of denominator of test statistic was used in the computation.")
-  } else{
-    frac.denom	<- sqrt(as.numeric(tu_m_outuu.m_o - 2* Matrix::crossprod(Matrix::t(tu.m_oX), Matrix::crossprod(Matrix::t(vcov.est), Matrix::crossprod(tZX, Matrix::tcrossprod(W.j, Matrix::t(tZutuu.m_o))))) + Matrix::crossprod(Matrix::t(tu.m_oX), Matrix::tcrossprod(vcov.est, tu.m_oX))))
+  } else {
+    frac.denom <- sqrt(frac.denom.sq)
   }
-
-  stat		<- frac.num/frac.denom
-  names(stat)	<- "normal"
-  pval		<- 2*stats::pnorm(abs(stat), lower.tail = FALSE)
-  mtest		<- list(statistic = stat, p.value = pval, method = paste("Arellano and Bond (1991) serial correlation test of degree", order)
-                 ,data.name = paste(object$iter, "step GMM Estimation", sep = "")
-                 ,alternative = paste("serial correlation of order ", order, " in the error terms", sep = "")
-  )
-  class(mtest)	<- "htest"
+  stat <- frac.num/frac.denom
+  names(stat) <- "normal"
+  pval <- 2 * stats::pnorm(abs(stat), lower.tail = FALSE)
+  mtest <- list(statistic = stat, p.value = pval,
+                method = paste("Arellano and Bond (1991) serial correlation test of degree",
+                              order), data.name = paste(object$iter, "step GMM Estimation", sep = ""),
+                alternative = paste("serial correlation of order ", order, " in the error terms", sep = ""))
+  class(mtest) <- "htest"
   return(mtest)
 }
 
